This is the mail archive of the gcc-patches@gcc.gnu.org mailing list for the GCC project.


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Other format: [Raw text]

[Bug middle-end/37448] patch to compute alloc-pool stats correctly


The df-ref problems pointed to by honza with respect to 37448 are
because alloc-pool.c incorrectly computed peak usage statistics.  It did
not compute them properly if pool_free was used to individually free
objects (like we do all over the place in df.)

This patch has been bootstapped on x86-64.

Note that no one owns alloc-pool so if someone with general oversight
could review this, it would be appreciated.

Note that with this patch and an enhancement to the debug dumps that i
am going to add to my previous patch, we now account for all of the
df-refs.  

OK to commit.

Kenny

2008-09-26  Kenneth Zadeck <zadeck@naturalbridge.com>

    PR rtl-optimization/37448
    alloc_pool_desc (elt_size): New field.
    alloc-pool.c (create_alloc_pool, empty_alloc_pool, pool_alloc,
    pool_free): Properly keep track of desc->size.
    (print_statistics, dump_alloc_pool_statistics): Enhance the
    printing of statistics to print the number of elements.

    

Index: alloc-pool.c
===================================================================
--- alloc-pool.c	(revision 140690)
+++ alloc-pool.c	(working copy)
@@ -64,14 +64,27 @@ static ALLOC_POOL_ID_TYPE last_id;
 
 #ifdef GATHER_STATISTICS
 
-/* Store information about each particular alloc_pool.  */
+/* Store information about each particular alloc_pool.  Note that this
+   will underestimate the amount the amount of storage used by a small amount:
+   1) The overhead in a pool is not accounted for.
+   2) The unallocated elements in a block are not accounted for.  Note
+   that this can at worst case be one element smaller that the block
+   size for that pool.
+*/
 struct alloc_pool_descriptor
 {
   const char *name;
-  int allocated;
+  /* Number of pools allocated.  */
   int created;
-  int peak;
+
+  /* Gross allocated storage.  */
+  int allocated;
+  /* Amount of currently active storage. */
   int current;
+  /* Peak amount of storage used.  */
+  int peak;
+  /* Size of element in the pool.  */
+  int elt_size;
 };
 
 /* Hashtable mapping alloc_pool names to descriptors.  */
@@ -150,6 +163,7 @@ create_alloc_pool (const char *name, siz
   pool->name = /*xstrdup (name)*/name;
 #ifdef GATHER_STATISTICS
   desc = alloc_pool_descriptor (name);
+  desc->elt_size = size;
   desc->created++;
 #endif
   pool->elt_size = size;
@@ -196,11 +210,11 @@ empty_alloc_pool (alloc_pool pool)
     {
       next_block = block->next;
       free (block);
-#ifdef GATHER_STATISTICS
-      desc->current -= pool->block_size;
-#endif
     }
 
+#ifdef GATHER_STATISTICS
+  desc->current -= (pool->elts_allocated - pool->elts_free) * pool->elt_size;
+#endif
   pool->returned_free_list = NULL;
   pool->virgin_free_list = NULL;
   pool->virgin_elts_remaining = 0;
@@ -242,7 +256,10 @@ pool_alloc (alloc_pool pool)
 #ifdef GATHER_STATISTICS
   struct alloc_pool_descriptor *desc = alloc_pool_descriptor (pool->name);
 
-  desc->allocated+=pool->elt_size;
+  desc->allocated += pool->elt_size;
+  desc->current += pool->elt_size;
+  if (desc->peak < desc->current)
+    desc->peak = desc->current;
 #endif
 
   gcc_assert (pool);
@@ -259,11 +276,6 @@ pool_alloc (alloc_pool pool)
 	  block = XNEWVEC (char, pool->block_size);
 	  block_header = (alloc_pool_list) block;
 	  block += align_eight (sizeof (struct alloc_pool_list_def));
-#ifdef GATHER_STATISTICS
-	  desc->current += pool->block_size;
-	  if (desc->peak < desc->current)
-	    desc->peak = desc->current;
-#endif
 	  
 	  /* Throw it on the block list.  */
 	  block_header->next = pool->block_list;
@@ -314,6 +326,9 @@ void
 pool_free (alloc_pool pool, void *ptr)
 {
   alloc_pool_list header;
+#ifdef GATHER_STATISTICS
+  struct alloc_pool_descriptor *desc = alloc_pool_descriptor (pool->name);
+#endif
 
   gcc_assert (ptr);
 
@@ -334,6 +349,11 @@ pool_free (alloc_pool pool, void *ptr)
   header->next = pool->returned_free_list;
   pool->returned_free_list = header;
   pool->elts_free++;
+
+#ifdef GATHER_STATISTICS
+  desc->current -= pool->elt_size;
+#endif
+
 }
 /* Output per-alloc_pool statistics.  */
 #ifdef GATHER_STATISTICS
@@ -355,8 +375,10 @@ print_statistics (void **slot, void *b)
 
   if (d->allocated)
     {
-      fprintf (stderr, "%-21s %6d %10d %10d %10d\n", d->name,
-	       d->created, d->allocated, d->peak, d->current);
+      fprintf (stderr, "%-22s %6d %10d %10d(%10d) %10d(%10d) %10d(%10d)\n", d->name,
+	       d->elt_size, d->created, d->allocated, d->allocated / d->elt_size, 
+	       d->peak, d->peak / d->elt_size, 
+	       d->current, d->current / d->elt_size);
       i->size += d->allocated;
       i->count += d->created;
     }
@@ -374,14 +396,14 @@ dump_alloc_pool_statistics (void)
   if (!alloc_pool_hash)
     return;
 
-  fprintf (stderr, "\nAlloc-pool Kind        Pools  Allocated      Peak        Leak\n");
-  fprintf (stderr, "-------------------------------------------------------------\n");
+  fprintf (stderr, "\nAlloc-pool Kind         Pools  Elt size  Allocated (elts)           Peak (elts)           Leak (elts)\n");
+  fprintf (stderr, "--------------------------------------------------------------------------------------------------------------\n");
   info.count = 0;
   info.size = 0;
   htab_traverse (alloc_pool_hash, print_statistics, &info);
-  fprintf (stderr, "-------------------------------------------------------------\n");
-  fprintf (stderr, "%-20s %7d %10d\n",
+  fprintf (stderr, "--------------------------------------------------------------------------------------------------------------\n");
+  fprintf (stderr, "%-22s           %7d %10d\n",
 	   "Total", info.count, info.size);
-  fprintf (stderr, "-------------------------------------------------------------\n");
+  fprintf (stderr, "--------------------------------------------------------------------------------------------------------------\n");
 #endif
 }

Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]